/*
Run simple wave dynamics

	Red (X) is X velocity
	Green (Y) is Y velocity
	Blue (Z) is height

Dr. Orion Sky Lawlor, lawlor@alaska.edu, 2011-03-22 (Public Domain)
*/
varying vec2 texCoords; /* interpolated from vertex shader */
uniform vec4 averages; /* overall average pixel values in srcTex */
uniform sampler2D srcTex; /* source texture */
const int dirs=4; /* automatically extracted by C++ */
uniform vec2 srcDirs[dirs]; /* texture coordinate shifts in each direction */

uniform float speed; /* =1.0     speed */
uniform float vel; /* =0.3   effect of height on velocity   */
uniform float Ht; /* =0.2  effect of velocity divergence on height */
uniform float velscale; /* =0.99 amount of viscous velocity loss per frame */
uniform float keynes; /* =0.1   amount of stimulus to inject (stable when <2.0) */

void main(void) /* fragment shader */ {
	vec4 C = texture2D(srcTex,texCoords); /* center pixel */
	vec4 R = texture2D(srcTex,texCoords+speed*srcDirs[0]);
	vec4 T = texture2D(srcTex,texCoords+srcDirs[1]);
	vec4 L = texture2D(srcTex,texCoords+srcDirs[2]);
	vec4 B = texture2D(srcTex,texCoords+srcDirs[3]);
	
	float XvelChange=L.z-R.z;
	float YvelChange=B.z-T.z;
	float HtChange=L.x-R.x+B.y-T.y;
	
	vec4 new=C;
	new.x+=vel*XvelChange;
	new.y+=vel*YvelChange;
	new.z+=Ht*HtChange;
	
	new.x*=velscale; new.y*=velscale;
	
	vec4 target=vec4(0.0,0.0,0.5,0.0);
	vec4 stimulus=target-averages;
	new=new+keynes*stimulus;
	
	gl_FragColor = vec4(clamp(vec3(new),-10.0,10.0),1.0);
}
